@@ -12,20 +12,18 @@
 #include "nsStringFwd.h"
 
 class nsIContent;
+class nsIDocument;
+struct JSContext;
 
 namespace mozilla {
 
 class ComputedTimingFunction;
 
-namespace dom {
-class Element;
-}
-
 class AnimationUtils
 {
 public:
   static dom::Nullable<double>
-    TimeDurationToDouble(const dom::Nullable<TimeDuration>& aTime)
+  TimeDurationToDouble(const dom::Nullable<TimeDuration>& aTime)
   {
     dom::Nullable<double> result;
 
@@ -37,7 +35,7 @@ public:
   }
 
   static dom::Nullable<TimeDuration>
-    DoubleToTimeDuration(const dom::Nullable<double>& aTime)
+  DoubleToTimeDuration(const dom::Nullable<double>& aTime)
   {
     dom::Nullable<TimeDuration> result;
 
@@ -52,12 +50,23 @@ public:
                                        const nsIContent* aContent = nullptr);
 
   /**
-   * Parses a CSS <single-transition-timing-function> value from
-   * aEasing into a ComputedTimingFunction.  If parsing fails, Nothing() will
-   * be returned.
+   * Get the document from the JS context to use when parsing CSS properties.
    */
-  static Maybe<ComputedTimingFunction>
-    ParseEasing(const dom::Element* aTarget, const nsAString& aEasing);
+  static nsIDocument*
+  GetCurrentRealmDocument(JSContext* aCx);
+
+  /**
+   * Checks if offscreen animation throttling is enabled.
+   */
+  static bool
+  IsOffscreenThrottlingEnabled();
+
+  /**
+   * Returns true if the preference to enable the core Web Animations API is
+   * true.
+   */
+  static bool
+  IsCoreAPIEnabled();
 };
 
 } // namespace mozilla
