@@ -30,6 +30,7 @@ class DOMStringList;
 class IDBCursor;
 class IDBRequest;
 class IDBTransaction;
+class StringOrStringSequence;
 template <typename> class Sequence;
 
 namespace indexedDB {
@@ -151,6 +152,9 @@ public:
     aName = Name();
   }
 
+  void
+  SetName(const nsAString& aName, ErrorResult& aRv);
+
   void
   GetKeyPath(JSContext* aCx, JS::MutableHandle<JS::Value> aResult,
              ErrorResult& aRv);
@@ -202,17 +206,11 @@ public:
   Get(JSContext* aCx, JS::Handle<JS::Value> aKey, ErrorResult& aRv);
 
   already_AddRefed<IDBRequest>
-  Clear(ErrorResult& aRv);
+  Clear(JSContext* aCx, ErrorResult& aRv);
 
   already_AddRefed<IDBIndex>
   CreateIndex(const nsAString& aName,
-              const nsAString& aKeyPath,
-              const IDBIndexParameters& aOptionalParameters,
-              ErrorResult& aRv);
-
-  already_AddRefed<IDBIndex>
-  CreateIndex(const nsAString& aName,
-              const Sequence<nsString>& aKeyPath,
+              const StringOrStringSequence& aKeyPath,
               const IDBIndexParameters& aOptionalParameters,
               ErrorResult& aRv);
 
@@ -261,6 +259,17 @@ public:
                               aRv);
   }
 
+  already_AddRefed<IDBRequest>
+  OpenCursor(JSContext* aCx,
+             IDBCursorDirection aDirection,
+             ErrorResult& aRv)
+  {
+    AssertIsOnOwningThread();
+
+    return OpenCursorInternal(/* aKeysOnly */ false, aCx,
+                              JS::UndefinedHandleValue, aDirection, aRv);
+  }
+
   already_AddRefed<IDBRequest>
   OpenKeyCursor(JSContext* aCx,
                 JS::Handle<JS::Value> aRange,
@@ -331,12 +340,6 @@ private:
                  const Optional<uint32_t>& aLimit,
                  ErrorResult& aRv);
 
-  already_AddRefed<IDBIndex>
-  CreateIndexInternal(const nsAString& aName,
-                      const KeyPath& aKeyPath,
-                      const IDBIndexParameters& aOptionalParameters,
-                      ErrorResult& aRv);
-
   already_AddRefed<IDBRequest>
   OpenCursorInternal(bool aKeysOnly,
                      JSContext* aCx,
