@@ -22,6 +22,7 @@
 #include "mozilla/Monitor.h"
 #include "mozilla/gfx/2D.h"
 #include "mozilla/layers/GrallocTextureClient.h"
+#include "nsAutoPtr.h"
 
 using namespace mozilla;
 using namespace mozilla::gfx;
@@ -413,6 +414,11 @@ ConvertGrallocImageToNV12(GrallocImage* aSource, uint8_t* aDestination)
 static nsresult
 ConvertSourceSurfaceToNV12(const RefPtr<SourceSurface>& aSurface, uint8_t* aDestination)
 {
+  if (!aSurface) {
+    CODEC_ERROR("Getting surface from image failed");
+    return NS_ERROR_FAILURE;
+  }
+
   uint32_t width = aSurface->GetSize().width;
   uint32_t height = aSurface->GetSize().height;
 
@@ -422,26 +428,19 @@ ConvertSourceSurfaceToNV12(const RefPtr<SourceSurface>& aSurface, uint8_t* aDest
   uint8_t* uv = y + (yStride * height);
   int uvStride = width / 2;
 
-  SurfaceFormat format = aSurface->GetFormat();
-
-  if (!aSurface) {
-    CODEC_ERROR("Getting surface %s from image failed", Stringify(format).c_str());
-    return NS_ERROR_FAILURE;
-  }
-
   RefPtr<DataSourceSurface> data = aSurface->GetDataSurface();
   if (!data) {
-    CODEC_ERROR("Getting data surface from %s image with %s (%s) surface failed",
-                Stringify(format).c_str(), Stringify(aSurface->GetType()).c_str(),
-                Stringify(aSurface->GetFormat()).c_str());
+    CODEC_ERROR("Getting data surface from %s image with %s surface failed",
+                Stringify(aSurface->GetFormat()).c_str(),
+                Stringify(aSurface->GetType()).c_str());
     return NS_ERROR_FAILURE;
   }
 
   DataSourceSurface::ScopedMap map(data, DataSourceSurface::READ);
   if (!map.IsMapped()) {
-    CODEC_ERROR("Reading DataSourceSurface from %s image with %s (%s) surface failed",
-                Stringify(format).c_str(), Stringify(aSurface->GetType()).c_str(),
-                Stringify(aSurface->GetFormat()).c_str());
+    CODEC_ERROR("Reading DataSourceSurface from %s image with %s surface failed",
+                Stringify(aSurface->GetFormat()).c_str(),
+                Stringify(aSurface->GetType()).c_str());
     return NS_ERROR_FAILURE;
   }
 
@@ -489,7 +488,7 @@ OMXVideoEncoder::Encode(const Image* aImage, int aWidth, int aHeight,
     NS_ENSURE_TRUE(aWidth == size.width, NS_ERROR_INVALID_ARG);
     NS_ENSURE_TRUE(aHeight == size.height, NS_ERROR_INVALID_ARG);
     if (format == ImageFormat::PLANAR_YCBCR) {
-      // Test for data, allowing SetDataNoCopy() on an image without an mBuffer
+      // Test for data, allowing AdoptData() on an image without an mBuffer
       // (as used from WebrtcOMXH264VideoCodec, and a few other places) - bug 1067442
       const PlanarYCbCrData* yuv = static_cast<PlanarYCbCrImage*>(img)->GetData();
       NS_ENSURE_TRUE(yuv->mYChannel, NS_ERROR_INVALID_ARG);
