@@ -8,13 +8,15 @@
 
 #include "nsISupports.h"
 #include "MediaResource.h"
+#include "GMPService.h"
 
 namespace mozilla {
 
 NS_IMPL_ISUPPORTS0(BufferDecoder)
 
-BufferDecoder::BufferDecoder(MediaResource* aResource)
+BufferDecoder::BufferDecoder(MediaResource* aResource, GMPCrashHelper* aCrashHelper)
   : mResource(aResource)
+  , mCrashHelper(aCrashHelper)
 {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_COUNT_CTOR(BufferDecoder);
@@ -33,13 +35,6 @@ BufferDecoder::BeginDecoding(TaskQueue* aTaskQueueIdentity)
   mTaskQueueIdentity = aTaskQueueIdentity;
 }
 
-bool
-BufferDecoder::OnStateMachineTaskQueue() const
-{
-  // BufferDecoder doesn't have the concept of a state machine.
-  return true;
-}
-
 MediaResource*
 BufferDecoder::GetResource() const
 {
@@ -47,20 +42,7 @@ BufferDecoder::GetResource() const
 }
 
 void
-BufferDecoder::NotifyBytesConsumed(int64_t aBytes, int64_t aOffset)
-{
-  // ignore
-}
-
-void
-BufferDecoder::NotifyDecodedFrames(uint32_t aParsed, uint32_t aDecoded,
-                                   uint32_t aDropped)
-{
-  // ignore
-}
-
-void
-BufferDecoder::SetMediaSeekable(bool aMediaSeekable)
+BufferDecoder::NotifyDecodedFrames(const FrameStatisticsData& aStats)
 {
   // ignore
 }
@@ -79,36 +61,6 @@ BufferDecoder::GetImageContainer()
   return nullptr;
 }
 
-bool
-BufferDecoder::IsTransportSeekable()
-{
-  return false;
-}
-
-bool
-BufferDecoder::IsMediaSeekable()
-{
-  return false;
-}
-
-void
-BufferDecoder::MetadataLoaded(nsAutoPtr<MediaInfo> aInfo, nsAutoPtr<MetadataTags> aTags, MediaDecoderEventVisibility aEventVisibility)
-{
-  // ignore
-}
-
-void
-BufferDecoder::FirstFrameLoaded(nsAutoPtr<MediaInfo> aInfo, MediaDecoderEventVisibility aEventVisibility)
-{
-  // ignore
-}
-
-void
-BufferDecoder::OnReadMetadataCompleted()
-{
-  // ignore
-}
-
 MediaDecoderOwner*
 BufferDecoder::GetOwner()
 {
@@ -116,4 +68,10 @@ BufferDecoder::GetOwner()
   return nullptr;
 }
 
+already_AddRefed<GMPCrashHelper>
+BufferDecoder::GetCrashHelper()
+{
+  return do_AddRef(mCrashHelper);
+}
+
 } // namespace mozilla
