@@ -113,8 +113,6 @@ parseUseAttrSets(txStylesheetAttr* aAttributes,
         NS_ENSURE_SUCCESS(rv, rv);
 
         nsAutoPtr<txInstruction> instr(new txInsertAttrSet(name));
-        NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
         rv = aState.addInstruction(Move(instr));
         NS_ENSURE_SUCCESS(rv, rv);
     }
@@ -191,7 +189,6 @@ getExprAttr(txStylesheetAttr* aAttributes,
                                     attr->mValue
 #endif
                                     );
-            NS_ENSURE_TRUE(aExpr, NS_ERROR_OUT_OF_MEMORY);
         }
         else {
             aExpr = nullptr;
@@ -228,7 +225,6 @@ getAVTAttr(txStylesheetAttr* aAttributes,
                                    attr->mValue
 #endif
                                    );
-            NS_ENSURE_TRUE(aAVT, NS_ERROR_OUT_OF_MEMORY);
         }
         else {
             aAVT = nullptr;
@@ -306,7 +302,7 @@ getAtomAttr(txStylesheetAttr* aAttributes,
         return rv;
     }
 
-    *aAtom = NS_NewAtom(attr->mValue).take();
+    *aAtom = NS_Atomize(attr->mValue).take();
     NS_ENSURE_TRUE(*aAtom, NS_ERROR_OUT_OF_MEMORY);
 
     return NS_OK;
@@ -525,12 +521,8 @@ txFnStartLREStylesheet(int32_t aNamespaceID,
     double prio = UnspecifiedNaN<double>();
 
     nsAutoPtr<txPattern> match(new txRootPattern());
-    NS_ENSURE_TRUE(match, NS_ERROR_OUT_OF_MEMORY);
-
     nsAutoPtr<txTemplateItem> templ(new txTemplateItem(Move(match), nullExpr,
                                                        nullExpr, prio));
-    NS_ENSURE_TRUE(templ, NS_ERROR_OUT_OF_MEMORY);
-
     aState.openInstructionContainer(templ);
     rv = aState.addToplevelItem(templ);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -553,8 +545,6 @@ txFnEndLREStylesheet(txStylesheetCompilerState& aState)
     aState.popHandlerTable();
 
     nsAutoPtr<txInstruction> instr(new txReturn());
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -638,8 +628,6 @@ txFnStartAttributeSet(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoPtr<txAttributeSetItem> attrSet(new txAttributeSetItem(name));
-    NS_ENSURE_TRUE(attrSet, NS_ERROR_OUT_OF_MEMORY);
-
     aState.openInstructionContainer(attrSet);
 
     rv = aState.addToplevelItem(attrSet);
@@ -659,8 +647,6 @@ txFnEndAttributeSet(txStylesheetCompilerState& aState)
     aState.popHandlerTable();
 
     nsAutoPtr<txInstruction> instr(new txReturn());
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -686,8 +672,6 @@ txFnStartDecimalFormat(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoPtr<txDecimalFormat> format(new txDecimalFormat);
-    NS_ENSURE_TRUE(format, NS_ERROR_OUT_OF_MEMORY);
-
     rv = getCharAttr(aAttributes, aAttrCount, nsGkAtoms::decimalSeparator,
                      false, aState, format->mDecimalSeparator);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -761,11 +745,7 @@ txFnStartImport(int32_t aNamespaceID,
                 txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txImportItem> import(new txImportItem);
-    NS_ENSURE_TRUE(import, NS_ERROR_OUT_OF_MEMORY);
-    
     import->mFrame = new txStylesheet::ImportFrame;
-    NS_ENSURE_TRUE(import->mFrame, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addToplevelItem(import);
     NS_ENSURE_SUCCESS(rv, rv);
     
@@ -910,7 +890,6 @@ txFnStartOutput(int32_t aNamespaceID,
     nsresult rv = NS_OK;
 
     nsAutoPtr<txOutputItem> item(new txOutputItem);
-    NS_ENSURE_TRUE(item, NS_ERROR_OUT_OF_MEMORY);
 
     txExpandedName methodExpName;
     rv = getQNameAttr(aAttributes, aAttrCount, nsGkAtoms::method, false,
@@ -977,8 +956,6 @@ txFnStartOutput(int32_t aNamespaceID,
         nsWhitespaceTokenizer tokens(attr->mValue);
         while (tokens.hasMoreTokens()) {
             nsAutoPtr<txExpandedName> qname(new txExpandedName());
-            NS_ENSURE_TRUE(qname, NS_ERROR_OUT_OF_MEMORY);
-
             rv = qname->init(tokens.nextToken(),
                              aState.mElementContext->mMappings, false);
             NS_ENSURE_SUCCESS(rv, rv);
@@ -1033,8 +1010,6 @@ txFnStartStripSpace(int32_t aNamespaceID,
     bool strip = aLocalName == nsGkAtoms::stripSpace;
 
     nsAutoPtr<txStripSpaceItem> stripItem(new txStripSpaceItem);
-    NS_ENSURE_TRUE(stripItem, NS_ERROR_OUT_OF_MEMORY);
-
     nsWhitespaceTokenizer tokenizer(attr->mValue);
     while (tokenizer.hasMoreTokens()) {
         const nsASingleFragmentString& name = tokenizer.nextToken();
@@ -1075,8 +1050,6 @@ txFnStartStripSpace(int32_t aNamespaceID,
         }
         nsAutoPtr<txStripSpaceTest> sst(new txStripSpaceTest(prefix, localName,
                                                              ns, strip));
-        NS_ENSURE_TRUE(sst, NS_ERROR_OUT_OF_MEMORY);
-
         rv = stripItem->addStripSpaceTest(sst);
         NS_ENSURE_SUCCESS(rv, rv);
         
@@ -1131,8 +1104,6 @@ txFnStartTemplate(int32_t aNamespaceID,
 
     nsAutoPtr<txTemplateItem> templ(new txTemplateItem(Move(match), name, mode,
                                                        prio));
-    NS_ENSURE_TRUE(templ, NS_ERROR_OUT_OF_MEMORY);
-
     aState.openInstructionContainer(templ);
     rv = aState.addToplevelItem(templ);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1148,8 +1119,6 @@ txFnEndTemplate(txStylesheetCompilerState& aState)
     aState.popHandlerTable();
 
     nsAutoPtr<txInstruction> instr(new txReturn());
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1181,8 +1150,6 @@ txFnStartTopVariable(int32_t aNamespaceID,
     nsAutoPtr<txVariableItem> var(
         new txVariableItem(name, Move(select),
                            aLocalName == nsGkAtoms::param));
-    NS_ENSURE_TRUE(var, NS_ERROR_OUT_OF_MEMORY);
-
     aState.openInstructionContainer(var);
     rv = aState.pushPtr(var, aState.eVariableItem);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1218,13 +1185,10 @@ txFnEndTopVariable(txStylesheetCompilerState& aState)
         NS_ASSERTION(!var->mValue,
                      "There shouldn't be a select-expression here");
         var->mValue = new txLiteralExpr(EmptyString());
-        NS_ENSURE_TRUE(var->mValue, NS_ERROR_OUT_OF_MEMORY);
     }
     else if (!var->mValue) {
         // If we don't have a select-expression there mush be children.
         nsAutoPtr<txInstruction> instr(new txReturn());
-        NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
         nsresult rv = aState.addInstruction(Move(instr));
         NS_ENSURE_SUCCESS(rv, rv);
     }
@@ -1282,8 +1246,6 @@ txFnStartLRE(int32_t aNamespaceID,
 
     nsAutoPtr<txInstruction> instr(new txStartLREElement(aNamespaceID,
                                                          aLocalName, aPrefix));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
     
@@ -1313,8 +1275,6 @@ txFnStartLRE(int32_t aNamespaceID,
 
         instr = new txLREAttribute(attr->mNamespaceID, attr->mLocalName,
                                    attr->mPrefix, Move(avt));
-        NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-        
         rv = aState.addInstruction(Move(instr));
         NS_ENSURE_SUCCESS(rv, rv);
     }
@@ -1326,8 +1286,6 @@ static nsresult
 txFnEndLRE(txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txInstruction> instr(new txEndElement);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1345,8 +1303,6 @@ txFnText(const nsAString& aStr, txStylesheetCompilerState& aState)
     TX_RETURN_IF_WHITESPACE(aStr, aState);
 
     nsAutoPtr<txInstruction> instr(new txText(aStr, false));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1370,14 +1326,10 @@ txFnStartApplyImports(int32_t aNamespaceID,
     nsresult rv = NS_OK;
 
     nsAutoPtr<txInstruction> instr(new txApplyImportsStart);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
     instr = new txApplyImportsEnd;
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1413,8 +1365,6 @@ txFnStartApplyTemplates(int32_t aNamespaceID,
     nsresult rv = NS_OK;
 
     nsAutoPtr<txInstruction> instr(new txPushParams);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1424,8 +1374,6 @@ txFnStartApplyTemplates(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     instr = new txApplyTemplates(mode);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushObject(instr);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1439,17 +1387,11 @@ txFnStartApplyTemplates(int32_t aNamespaceID,
     if (!select) {
         nsAutoPtr<txNodeTest> nt(
             new txNodeTypeTest(txNodeTypeTest::NODE_TYPE));
-        NS_ENSURE_TRUE(nt, NS_ERROR_OUT_OF_MEMORY);
-
         select = new LocationStep(nt, LocationStep::CHILD_AXIS);
-        NS_ENSURE_TRUE(select, NS_ERROR_OUT_OF_MEMORY);
-
         nt.forget();
     }
 
     nsAutoPtr<txPushNewContext> pushcontext( new txPushNewContext(Move(select)));
-    NS_ENSURE_TRUE(pushcontext, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushSorter(pushcontext);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1476,8 +1418,6 @@ txFnEndApplyTemplates(txStylesheetCompilerState& aState)
 
     instr = static_cast<txInstruction*>(aState.popObject()); // txApplyTemplates
     nsAutoPtr<txLoopNodeSet> loop(new txLoopNodeSet(instr));
-    NS_ENSURE_TRUE(loop, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1486,8 +1426,6 @@ txFnEndApplyTemplates(txStylesheetCompilerState& aState)
     NS_ENSURE_SUCCESS(rv, rv);
 
     instr = new txPopParams;
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-    
     pushcontext->mBailTarget = instr;
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1513,8 +1451,6 @@ txFnStartAttribute(int32_t aNamespaceID,
     nsresult rv = NS_OK;
 
     nsAutoPtr<txInstruction> instr(new txPushStringHandler(true));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1530,8 +1466,6 @@ txFnStartAttribute(int32_t aNamespaceID,
 
     instr = new txAttribute(Move(name), Move(nspace),
                             aState.mElementContext->mMappings);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushObject(instr);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1573,8 +1507,6 @@ txFnStartCallTemplate(int32_t aNamespaceID,
     nsresult rv = NS_OK;
 
     nsAutoPtr<txInstruction> instr(new txPushParams);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1584,8 +1516,6 @@ txFnStartCallTemplate(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     instr = new txCallTemplate(name);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushObject(instr);
     NS_ENSURE_SUCCESS(rv, rv);
     
@@ -1605,8 +1535,6 @@ txFnEndCallTemplate(txStylesheetCompilerState& aState)
     NS_ENSURE_SUCCESS(rv, rv);
 
     instr = new txPopParams;
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1674,8 +1602,6 @@ txFnStartComment(int32_t aNamespaceID,
                  txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txInstruction> instr(new txPushStringHandler(true));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1686,8 +1612,6 @@ static nsresult
 txFnEndComment(txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txInstruction> instr(new txComment);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1712,8 +1636,6 @@ txFnStartCopy(int32_t aNamespaceID,
               txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txCopy> copy(new txCopy);
-    NS_ENSURE_TRUE(copy, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.pushPtr(copy, aState.eCopy);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1731,8 +1653,6 @@ static nsresult
 txFnEndCopy(txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txInstruction> instr(new txEndElement);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1764,8 +1684,6 @@ txFnStartCopyOf(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoPtr<txInstruction> instr(new txCopyOf(Move(select)));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-    
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1810,8 +1728,6 @@ txFnStartElement(int32_t aNamespaceID,
     nsAutoPtr<txInstruction> instr(
         new txStartElement(Move(name), Move(nspace),
                            aState.mElementContext->mMappings));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1825,8 +1741,6 @@ static nsresult
 txFnEndElement(txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txInstruction> instr(new txEndElement);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1886,8 +1800,6 @@ txFnStartForEach(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoPtr<txPushNewContext> pushcontext(new txPushNewContext(Move(select)));
-    NS_ENSURE_TRUE(pushcontext, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushPtr(pushcontext, aState.ePushNewContext);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1899,8 +1811,6 @@ txFnStartForEach(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
     
     instr = new txPushNullTemplateRule;
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushPtr(instr, aState.ePushNullTemplateRule);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1979,8 +1889,6 @@ txFnStartIf(int32_t aNamespaceID,
 
     nsAutoPtr<txConditionalGoto> condGoto(new txConditionalGoto(Move(test),
                                                                 nullptr));
-    NS_ENSURE_TRUE(condGoto, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushPtr(condGoto, aState.eConditionalGoto);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2015,8 +1923,6 @@ txFnStartMessage(int32_t aNamespaceID,
                  txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txInstruction> instr(new txPushStringHandler(false));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2026,8 +1932,6 @@ txFnStartMessage(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     instr = new txMessage(term == eTrue);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushObject(instr);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2121,8 +2025,6 @@ txFnStartNumber(int32_t aNamespaceID,
                                                 Move(value), Move(format),
                                                 Move(groupingSeparator),
                                                 Move(groupingSize)));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-    
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2202,8 +2104,6 @@ txFnStartParam(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoPtr<txSetVariable> var(new txSetVariable(name, Move(select)));
-    NS_ENSURE_TRUE(var, NS_ERROR_OUT_OF_MEMORY);
-
     if (var->mValue) {
         // XXX should be gTxErrorHandler?
         rv = aState.pushHandlerTable(gTxIgnoreHandler);
@@ -2235,7 +2135,6 @@ txFnEndParam(txStylesheetCompilerState& aState)
         NS_ASSERTION(!var->mValue,
                      "There shouldn't be a select-expression here");
         var->mValue = new txLiteralExpr(EmptyString());
-        NS_ENSURE_TRUE(var->mValue, NS_ERROR_OUT_OF_MEMORY);
     }
 
     nsresult rv = aState.addVariable(var->mName);
@@ -2268,8 +2167,6 @@ txFnStartPI(int32_t aNamespaceID,
             txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txInstruction> instr(new txPushStringHandler(true));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2279,8 +2176,6 @@ txFnStartPI(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     instr = new txProcessingInstruction(Move(name));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushObject(instr);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2323,10 +2218,8 @@ txFnStartSort(int32_t aNamespaceID,
     if (!select) {
         nsAutoPtr<txNodeTest> nt(
               new txNodeTypeTest(txNodeTypeTest::NODE_TYPE));
-        NS_ENSURE_TRUE(nt, NS_ERROR_OUT_OF_MEMORY);
 
         select = new LocationStep(nt, LocationStep::SELF_AXIS);
-        NS_ENSURE_TRUE(select, NS_ERROR_OUT_OF_MEMORY);
 
         nt.forget();
     }
@@ -2405,8 +2298,6 @@ static nsresult
 txFnTextText(const nsAString& aStr, txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txInstruction> instr(new txText(aStr, aState.mDOE));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2440,8 +2331,6 @@ txFnStartValueOf(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoPtr<txInstruction> instr(new txValueOf(Move(select), doe == eTrue));
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2483,8 +2372,6 @@ txFnStartVariable(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoPtr<txSetVariable> var(new txSetVariable(name, Move(select)));
-    NS_ENSURE_TRUE(var, NS_ERROR_OUT_OF_MEMORY);
-
     if (var->mValue) {
         // XXX should be gTxErrorHandler?
         rv = aState.pushHandlerTable(gTxIgnoreHandler);
@@ -2517,7 +2404,6 @@ txFnEndVariable(txStylesheetCompilerState& aState)
         NS_ASSERTION(!var->mValue,
                      "There shouldn't be a select-expression here");
         var->mValue = new txLiteralExpr(EmptyString());
-        NS_ENSURE_TRUE(var->mValue, NS_ERROR_OUT_OF_MEMORY);
     }
 
     nsresult rv = aState.addVariable(var->mName);
@@ -2539,8 +2425,6 @@ txFnStartElementStartRTF(int32_t aNamespaceID,
                          txStylesheetCompilerState& aState)
 {
     nsAutoPtr<txInstruction> instr(new txPushRTFHandler);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2555,8 +2439,6 @@ txFnTextStartRTF(const nsAString& aStr, txStylesheetCompilerState& aState)
     TX_RETURN_IF_WHITESPACE(aStr, aState);
 
     nsAutoPtr<txInstruction> instr(new txPushRTFHandler);
-    NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
     nsresult rv = aState.addInstruction(Move(instr));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2587,8 +2469,6 @@ txFnStartWhen(int32_t aNamespaceID,
 
     nsAutoPtr<txConditionalGoto> condGoto(new txConditionalGoto(Move(test),
                                                                 nullptr));
-    NS_ENSURE_TRUE(condGoto, NS_ERROR_OUT_OF_MEMORY);
-
     rv = aState.pushPtr(condGoto, aState.eConditionalGoto);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2604,8 +2484,6 @@ txFnEndWhen(txStylesheetCompilerState& aState)
 {
     aState.popHandlerTable();
     nsAutoPtr<txGoTo> gotoinstr(new txGoTo(nullptr));
-    NS_ENSURE_TRUE(gotoinstr, NS_ERROR_OUT_OF_MEMORY);
-    
     nsresult rv = aState.mChooseGotoList->add(gotoinstr);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2649,8 +2527,6 @@ txFnStartWithParam(int32_t aNamespaceID,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoPtr<txSetParam> var(new txSetParam(name, Move(select)));
-    NS_ENSURE_TRUE(var, NS_ERROR_OUT_OF_MEMORY);
-
     if (var->mValue) {
         // XXX should be gTxErrorHandler?
         rv = aState.pushHandlerTable(gTxIgnoreHandler);
@@ -2681,7 +2557,6 @@ txFnEndWithParam(txStylesheetCompilerState& aState)
         NS_ASSERTION(!var->mValue,
                      "There shouldn't be a select-expression here");
         var->mValue = new txLiteralExpr(EmptyString());
-        NS_ENSURE_TRUE(var->mValue, NS_ERROR_OUT_OF_MEMORY);
     }
 
     nsAutoPtr<txInstruction> instr(var.forget());
@@ -2725,8 +2600,6 @@ txFnEndUnknownInstruction(txStylesheetCompilerState& aState)
 
     if (aState.mSearchingForFallback) {
         nsAutoPtr<txInstruction> instr(new txErrorInstruction());
-        NS_ENSURE_TRUE(instr, NS_ERROR_OUT_OF_MEMORY);
-
         nsresult rv = aState.addInstruction(Move(instr));
         NS_ENSURE_SUCCESS(rv, rv);
     }
@@ -2986,7 +2859,7 @@ txHandlerTable::init(const txElementHandler* aHandlers, uint32_t aCount)
 
     uint32_t i;
     for (i = 0; i < aCount; ++i) {
-        nsCOMPtr<nsIAtom> nameAtom = do_GetAtom(aHandlers->mLocalName);
+        nsCOMPtr<nsIAtom> nameAtom = NS_Atomize(aHandlers->mLocalName);
         txExpandedName name(aHandlers->mNamespaceID, nameAtom);
         rv = mHandlers.add(name, aHandlers);
         NS_ENSURE_SUCCESS(rv, rv);
