@@ -1,3 +1,5 @@
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 // Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
@@ -12,6 +14,32 @@
 #include "base/string16.h"
 
 #include "mozilla/Attributes.h"
+#include "mozilla/BufferList.h"
+#include "mozilla/mozalloc.h"
+
+#ifdef MOZ_FAULTY
+#include "base/singleton.h"
+#include "mozilla/ipc/Faulty.h"
+#endif
+
+#if !defined(RELEASE_BUILD) || defined(DEBUG)
+#define MOZ_PICKLE_SENTINEL_CHECKING
+#endif
+
+class Pickle;
+
+class PickleIterator {
+public:
+  explicit PickleIterator(const Pickle& pickle);
+
+private:
+  friend class Pickle;
+
+  mozilla::BufferList<InfallibleAllocPolicy>::IterImpl iter_;
+
+  template<typename T>
+  void CopyInto(T* dest);
+};
 
 // This class provides facilities for basic binary value packing and unpacking.
 //
@@ -32,125 +60,155 @@
 //
 class Pickle {
  public:
-  enum Ownership {
-    BORROWS,
-    OWNS,
-  };
-
   ~Pickle();
 
-  // Initialize a Pickle object using the default header size.
-  Pickle();
+  Pickle() = delete;
 
   // Initialize a Pickle object with the specified header size in bytes, which
   // must be greater-than-or-equal-to sizeof(Pickle::Header).  The header size
   // will be rounded up to ensure that the header size is 32bit-aligned.
-  explicit Pickle(int header_size);
+  explicit Pickle(uint32_t header_size);
 
-  // Initializes a Pickle from a const block of data. If ownership == BORROWS,
-  // the data is not copied; instead the data is merely referenced by this
-  // Pickle. Only const methods should be used on the Pickle when initialized
-  // this way. The header padding size is deduced from the data length.  If
-  // ownership == OWNS, then again no copying takes place. However, the buffer
-  // is writable and will be freed when this Pickle is destroyed.
-  Pickle(const char* data, int data_len, Ownership ownership = BORROWS);
+  Pickle(uint32_t header_size, const char* data, uint32_t length);
 
-  // Initializes a Pickle as a deep copy of another Pickle.
-  Pickle(const Pickle& other);
+  Pickle(const Pickle& other) = delete;
 
   Pickle(Pickle&& other);
 
   // Performs a deep copy.
-  Pickle& operator=(const Pickle& other);
+  Pickle& operator=(const Pickle& other) = delete;
 
   Pickle& operator=(Pickle&& other);
 
   // Returns the size of the Pickle's data.
-  int size() const { return static_cast<int>(header_size_ +
-                                             header_->payload_size); }
+  uint32_t size() const { return header_size_ + header_->payload_size; }
 
-  // Return the full size of the memory allocated for this Pickle's data.
-  uint32_t capacity() const {
-    return capacity_;
-  }
+  typedef mozilla::BufferList<InfallibleAllocPolicy> BufferList;
 
-  // Returns the data for this Pickle.
-  const void* data() const { return header_; }
+  const BufferList& Buffers() const { return buffers_; }
+
+  uint32_t CurrentSize() const { return buffers_.Size(); }
 
   // Methods for reading the payload of the Pickle.  To read from the start of
   // the Pickle, initialize *iter to NULL.  If successful, these methods return
   // true.  Otherwise, false is returned to indicate that the result could not
   // be extracted.
-  MOZ_WARN_UNUSED_RESULT bool ReadBool(void** iter, bool* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadInt16(void** iter, int16_t* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadUInt16(void** iter, uint16_t* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadShort(void** iter, short* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadInt(void** iter, int* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadLong(void** iter, long* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadULong(void** iter, unsigned long* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadSize(void** iter, size_t* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadInt32(void** iter, int32_t* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadUInt32(void** iter, uint32_t* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadInt64(void** iter, int64_t* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadUInt64(void** iter, uint64_t* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadDouble(void** iter, double* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadIntPtr(void** iter, intptr_t* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadUnsignedChar(void** iter, unsigned char* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadString(void** iter, std::string* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadWString(void** iter, std::wstring* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadString16(void** iter, string16* result) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadData(void** iter, const char** data, int* length) const;
-  MOZ_WARN_UNUSED_RESULT bool ReadBytes(void** iter, const char** data, int length,
-					uint32_t alignment = sizeof(memberAlignmentType)) const;
+  MOZ_MUST_USE bool ReadBool(PickleIterator* iter, bool* result) const;
+  MOZ_MUST_USE bool ReadInt16(PickleIterator* iter, int16_t* result) const;
+  MOZ_MUST_USE bool ReadUInt16(PickleIterator* iter, uint16_t* result) const;
+  MOZ_MUST_USE bool ReadShort(PickleIterator* iter, short* result) const;
+  MOZ_MUST_USE bool ReadInt(PickleIterator* iter, int* result) const;
+  MOZ_MUST_USE bool ReadLong(PickleIterator* iter, long* result) const;
+  MOZ_MUST_USE bool ReadULong(PickleIterator* iter, unsigned long* result) const;
+  MOZ_MUST_USE bool ReadSize(PickleIterator* iter, size_t* result) const;
+  MOZ_MUST_USE bool ReadInt32(PickleIterator* iter, int32_t* result) const;
+  MOZ_MUST_USE bool ReadUInt32(PickleIterator* iter, uint32_t* result) const;
+  MOZ_MUST_USE bool ReadInt64(PickleIterator* iter, int64_t* result) const;
+  MOZ_MUST_USE bool ReadUInt64(PickleIterator* iter, uint64_t* result) const;
+  MOZ_MUST_USE bool ReadDouble(PickleIterator* iter, double* result) const;
+  MOZ_MUST_USE bool ReadIntPtr(PickleIterator* iter, intptr_t* result) const;
+  MOZ_MUST_USE bool ReadUnsignedChar(PickleIterator* iter, unsigned char* result) const;
+  MOZ_MUST_USE bool ReadString(PickleIterator* iter, std::string* result) const;
+  MOZ_MUST_USE bool ReadWString(PickleIterator* iter, std::wstring* result) const;
+  MOZ_MUST_USE bool ReadBytesInto(PickleIterator* iter, void* data, uint32_t length) const;
+  MOZ_MUST_USE bool FlattenBytes(PickleIterator* iter, const char** data, uint32_t length,
+                                 uint32_t alignment = sizeof(memberAlignmentType));
 
   // Safer version of ReadInt() checks for the result not being negative.
   // Use it for reading the object sizes.
-  MOZ_WARN_UNUSED_RESULT bool ReadLength(void** iter, int* result) const;
+  MOZ_MUST_USE bool ReadLength(PickleIterator* iter, int* result) const;
+
+  MOZ_MUST_USE bool ReadSentinel(PickleIterator* iter, uint32_t sentinel) const
+#ifdef MOZ_PICKLE_SENTINEL_CHECKING
+    ;
+#else
+  {
+    return true;
+  }
+#endif
+
+  void EndRead(PickleIterator& iter) const;
 
   // Methods for adding to the payload of the Pickle.  These values are
   // appended to the end of the Pickle's payload.  When reading values from a
   // Pickle, it is important to read them in the order in which they were added
   // to the Pickle.
   bool WriteBool(bool value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzBool(&value);
+#endif
     return WriteInt(value ? 1 : 0);
   }
   bool WriteInt16(int16_t value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzInt16(&value);
+#endif
     return WriteBytes(&value, sizeof(value));
   }
   bool WriteUInt16(uint16_t value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzUInt16(&value);
+#endif
     return WriteBytes(&value, sizeof(value));
   }
   bool WriteInt(int value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzInt(&value);
+#endif
     return WriteBytes(&value, sizeof(value));
   }
   bool WriteLong(long value) {
     // Always written as a 64-bit value since the size for this type can
     // differ between architectures.
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzLong(&value);
+#endif
     return WriteInt64(int64_t(value));
   }
   bool WriteULong(unsigned long value) {
     // Always written as a 64-bit value since the size for this type can
     // differ between architectures.
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzULong(&value);
+#endif
     return WriteUInt64(uint64_t(value));
   }
   bool WriteSize(size_t value) {
     // Always written as a 64-bit value since the size for this type can
     // differ between architectures.
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzSize(&value);
+#endif
     return WriteUInt64(uint64_t(value));
   }
   bool WriteInt32(int32_t value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzInt(&value);
+#endif
     return WriteBytes(&value, sizeof(value));
   }
   bool WriteUInt32(uint32_t value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzUInt32(&value);
+#endif
     return WriteBytes(&value, sizeof(value));
   }
   bool WriteInt64(int64_t value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzInt64(&value);
+#endif
     return WriteBytes(&value, sizeof(value));
   }
   bool WriteUInt64(uint64_t value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzUInt64(&value);
+#endif
     return WriteBytes(&value, sizeof(value));
   }
   bool WriteDouble(double value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzDouble(&value);
+#endif
     return WriteBytes(&value, sizeof(value));
   }
   bool WriteIntPtr(intptr_t value) {
@@ -159,39 +217,29 @@ class Pickle {
     return WriteInt64(int64_t(value));
   }
   bool WriteUnsignedChar(unsigned char value) {
+#ifdef MOZ_FAULTY
+    Singleton<mozilla::ipc::Faulty>::get()->FuzzUChar(&value);
+#endif
     return WriteBytes(&value, sizeof(value));
   }
   bool WriteString(const std::string& value);
   bool WriteWString(const std::wstring& value);
-  bool WriteString16(const string16& value);
-  bool WriteData(const char* data, int length);
-  bool WriteBytes(const void* data, int data_len,
+  bool WriteData(const char* data, uint32_t length);
+  bool WriteBytes(const void* data, uint32_t data_len,
                   uint32_t alignment = sizeof(memberAlignmentType));
 
-  // Same as WriteData, but allows the caller to write directly into the
-  // Pickle. This saves a copy in cases where the data is not already
-  // available in a buffer. The caller should take care to not write more
-  // than the length it declares it will. Use ReadData to get the data.
-  // Returns NULL on failure.
-  //
-  // The returned pointer will only be valid until the next write operation
-  // on this Pickle.
-  char* BeginWriteData(int length);
-
-  // For Pickles which contain variable length buffers (e.g. those created
-  // with BeginWriteData), the Pickle can
-  // be 'trimmed' if the amount of data required is less than originally
-  // requested.  For example, you may have created a buffer with 10K of data,
-  // but decided to only fill 10 bytes of that data.  Use this function
-  // to trim the buffer so that we don't send 9990 bytes of unused data.
-  // You cannot increase the size of the variable buffer; only shrink it.
-  // This function assumes that the length of the variable buffer has
-  // not been changed.
-  void TrimWriteData(int length);
-
-  void EndRead(void* iter) const {
-    DCHECK(iter == end_of_payload());
+  bool WriteSentinel(uint32_t sentinel)
+#ifdef MOZ_PICKLE_SENTINEL_CHECKING
+    ;
+#else
+  {
+    return true;
   }
+#endif
+
+  int32_t* GetInt32PtrForTest(uint32_t offset);
+
+  void InputBytes(const char* data, uint32_t length);
 
   // Payload follows after allocation of Header (header size is customizable).
   struct Header {
@@ -212,63 +260,28 @@ class Pickle {
     return static_cast<const T*>(header_);
   }
 
-  // Returns true if the given iterator could point to data with the given
-  // length. If there is no room for the given data before the end of the
-  // payload, returns false.
-  bool IteratorHasRoomFor(const void* iter, int len) const {
-    if ((len < 0) || (iter < header_) || iter > end_of_payload())
-      return false;
-    const char* end_of_region = reinterpret_cast<const char*>(iter) + len;
-    // Watch out for overflow in pointer calculation, which wraps.
-    return (iter <= end_of_region) && (end_of_region <= end_of_payload());
-  }
-
   typedef uint32_t memberAlignmentType;
 
  protected:
   uint32_t payload_size() const { return header_->payload_size; }
 
-  char* payload() {
-    return reinterpret_cast<char*>(header_) + header_size_;
-  }
-  const char* payload() const {
-    return reinterpret_cast<const char*>(header_) + header_size_;
-  }
-
-  // Returns the address of the byte immediately following the currently valid
-  // header + payload.
-  char* end_of_payload() {
-    // We must have a valid header_.
-    return payload() + payload_size();
-  }
-  const char* end_of_payload() const {
-    // This object may be invalid.
-    return header_ ? payload() + payload_size() : nullptr;
-  }
-
   // Resizes the buffer for use when writing the specified amount of data. The
   // location that the data should be written at is returned, or NULL if there
   // was an error. Call EndWrite with the returned offset and the given length
   // to pad out for the next write.
-  char* BeginWrite(uint32_t length, uint32_t alignment);
+  void BeginWrite(uint32_t length, uint32_t alignment);
 
   // Completes the write operation by padding the data with NULL bytes until it
   // is padded. Should be paired with BeginWrite, but it does not necessarily
   // have to be called after the data is written.
-  void EndWrite(char* dest, int length);
-
-  // Resize the capacity, note that the input value should include the size of
-  // the header: new_capacity = sizeof(Header) + desired_payload_capacity.
-  // A realloc() failure will cause a Resize failure... and caller should check
-  // the return result for true (i.e., successful resizing).
-  void Resize(uint32_t new_capacity);
+  void EndWrite(uint32_t length);
 
   // Round 'bytes' up to the next multiple of 'alignment'.  'alignment' must be
   // a power of 2.
   template<uint32_t alignment> struct ConstantAligner {
     static uint32_t align(int bytes) {
       static_assert((alignment & (alignment - 1)) == 0,
-			"alignment must be a power of two");
+                    "alignment must be a power of two");
       return (bytes + (alignment - 1)) & ~static_cast<uint32_t>(alignment - 1);
     }
   };
@@ -277,33 +290,37 @@ class Pickle {
     return ConstantAligner<sizeof(memberAlignmentType)>::align(bytes);
   }
 
+  static uint32_t AlignCapacity(int bytes) {
+    return ConstantAligner<kSegmentAlignment>::align(bytes);
+  }
+
+  // Returns true if the given iterator could point to data with the given
+  // length. If there is no room for the given data before the end of the
+  // payload, returns false.
+  bool IteratorHasRoomFor(const PickleIterator& iter, uint32_t len) const;
+
   // Moves the iterator by the given number of bytes, making sure it is aligned.
   // Pointer (iterator) is NOT aligned, but the change in the pointer
   // is guaranteed to be a multiple of sizeof(memberAlignmentType).
-  static void UpdateIter(void** iter, int bytes) {
-    *iter = static_cast<char*>(*iter) + AlignInt(bytes);
-  }
+  void UpdateIter(PickleIterator* iter, uint32_t bytes) const;
 
-  // Find the end of the pickled data that starts at range_start.  Returns NULL
-  // if the entire Pickle is not found in the given data range.
-  static const char* FindNext(uint32_t header_size,
+  // Figure out how big the message starting at range_start is. Returns 0 if
+  // there's no enough data to determine (i.e., if [range_start, range_end) does
+  // not contain enough of the message header to know the size).
+  static uint32_t MessageSize(uint32_t header_size,
                               const char* range_start,
                               const char* range_end);
 
-  // If the given range contains at least header_size bytes, return the length
-  // of the pickled data including the header.
-  static uint32_t GetLength(uint32_t header_size,
-                            const char* range_start,
-                            const char* range_end);
-
-  // The allocation granularity of the payload.
-  static const int kPayloadUnit;
+  // Segments capacities are aligned to 8 bytes to ensure that all reads/writes
+  // at 8-byte aligned offsets will be on 8-byte aligned pointers.
+  static const uint32_t kSegmentAlignment = 8;
 
  private:
+  friend class PickleIterator;
+
+  BufferList buffers_;
   Header* header_;
   uint32_t header_size_;
-  uint32_t capacity_;
-  uint32_t variable_buffer_offset_;
 };
 
 #endif  // BASE_PICKLE_H__
