@@ -29,7 +29,6 @@
 #include "nsIDOMMutationEvent.h"
 #include "nsNameSpaceManager.h"
 #include "nsCSSAnonBoxes.h"
-#include "nsAutoPtr.h"
 #include "mozilla/StyleSetHandle.h"
 #include "mozilla/StyleSetHandleInlines.h"
 #include "mozilla/dom/Element.h"
@@ -105,8 +104,8 @@ public:
                                 const nsDisplayListSet& aLists) override;
 
   virtual void Reflow(nsPresContext*           aPresContext,
-                          nsHTMLReflowMetrics&     aDesiredSize,
-                          const nsHTMLReflowState& aReflowState,
+                          ReflowOutput&     aDesiredSize,
+                          const ReflowInput& aReflowInput,
                           nsReflowStatus&          aStatus) override;
 
   bool GetVisibility() { return mVisibility; }
@@ -154,8 +153,8 @@ public:
                                 const nsDisplayListSet& aLists) override;
 
   virtual void Reflow(nsPresContext*           aPresContext,
-                          nsHTMLReflowMetrics&     aDesiredSize,
-                          const nsHTMLReflowState& aReflowState,
+                          ReflowOutput&     aDesiredSize,
+                          const ReflowInput& aReflowInput,
                           nsReflowStatus&          aStatus) override;
 
 protected:
@@ -280,9 +279,7 @@ nsHTMLFramesetFrame::Init(nsIContent*       aContent,
       for (uint32_t i = childX; i < numChildren; i++) {
         nsIContent *child = mContent->GetChildAt(i);
         child->UnsetFlags(NODE_DESCENDANTS_NEED_FRAMES | NODE_NEEDS_FRAME);
-        if (child->IsElement()) {
-          child->UnsetFlags(ELEMENT_ALL_RESTYLE_FLAGS);
-        }
+        child->UnsetRestyleFlagsIfGecko();
       }
       break;
     }
@@ -290,9 +287,7 @@ nsHTMLFramesetFrame::Init(nsIContent*       aContent,
     child->UnsetFlags(NODE_DESCENDANTS_NEED_FRAMES | NODE_NEEDS_FRAME);
     // Also clear the restyle flags in the child like
     // nsCSSFrameConstructor::ProcessChildren does.
-    if (child->IsElement()) {
-      child->UnsetFlags(ELEMENT_ALL_RESTYLE_FLAGS);
-    }
+    child->UnsetRestyleFlagsIfGecko();
 
     // IMPORTANT: This must match the conditions in
     // nsCSSFrameConstructor::ContentAppended/Inserted/Removed
@@ -568,17 +563,17 @@ int32_t nsHTMLFramesetFrame::GetBorderWidth(nsPresContext* aPresContext,
 
 void
 nsHTMLFramesetFrame::GetDesiredSize(nsPresContext*           aPresContext,
-                                    const nsHTMLReflowState& aReflowState,
-                                    nsHTMLReflowMetrics&     aDesiredSize)
+                                    const ReflowInput& aReflowInput,
+                                    ReflowOutput&     aDesiredSize)
 {
-  WritingMode wm = aReflowState.GetWritingMode();
+  WritingMode wm = aReflowInput.GetWritingMode();
   LogicalSize desiredSize(wm);
   nsHTMLFramesetFrame* framesetParent = do_QueryFrame(GetParent());
   if (nullptr == framesetParent) {
     if (aPresContext->IsPaginated()) {
       // XXX This needs to be changed when framesets paginate properly
-      desiredSize.ISize(wm) = aReflowState.AvailableISize();
-      desiredSize.BSize(wm) = aReflowState.AvailableBSize();
+      desiredSize.ISize(wm) = aReflowInput.AvailableISize();
+      desiredSize.BSize(wm) = aReflowInput.AvailableBSize();
     } else {
       LogicalSize area(wm, aPresContext->GetVisibleArea().Size());
 
@@ -643,12 +638,12 @@ nsresult nsHTMLFramesetFrame::HandleEvent(nsPresContext* aPresContext,
     switch (aEvent->mMessage) {
       case eMouseMove:
         MouseDrag(aPresContext, aEvent);
-	      break;
+        break;
       case eMouseUp:
         if (aEvent->AsMouseEvent()->button == WidgetMouseEvent::eLeftButton) {
           EndMouseDrag(aPresContext);
         }
-	      break;
+        break;
       default:
         break;
     }
@@ -687,29 +682,29 @@ nsHTMLFramesetFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
 void
 nsHTMLFramesetFrame::ReflowPlaceChild(nsIFrame*                aChild,
                                       nsPresContext*           aPresContext,
-                                      const nsHTMLReflowState& aReflowState,
+                                      const ReflowInput& aReflowInput,
                                       nsPoint&                 aOffset,
                                       nsSize&                  aSize,
                                       nsIntPoint*              aCellIndex)
 {
   // reflow the child
-  nsHTMLReflowState reflowState(aPresContext, aReflowState, aChild,
+  ReflowInput reflowInput(aPresContext, aReflowInput, aChild,
                                 LogicalSize(aChild->GetWritingMode(), aSize));
-  reflowState.SetComputedWidth(std::max(0, aSize.width - reflowState.ComputedPhysicalBorderPadding().LeftRight()));
-  reflowState.SetComputedHeight(std::max(0, aSize.height - reflowState.ComputedPhysicalBorderPadding().TopBottom()));
-  nsHTMLReflowMetrics metrics(aReflowState);
-  metrics.Width() = aSize.width;
-  metrics.Height() = aSize.height;
+  reflowInput.SetComputedWidth(std::max(0, aSize.width - reflowInput.ComputedPhysicalBorderPadding().LeftRight()));
+  reflowInput.SetComputedHeight(std::max(0, aSize.height - reflowInput.ComputedPhysicalBorderPadding().TopBottom()));
+  ReflowOutput reflowOutput(aReflowInput);
+  reflowOutput.Width() = aSize.width;
+  reflowOutput.Height() = aSize.height;
   nsReflowStatus status;
 
-  ReflowChild(aChild, aPresContext, metrics, reflowState, aOffset.x,
+  ReflowChild(aChild, aPresContext, reflowOutput, reflowInput, aOffset.x,
               aOffset.y, 0, status);
   NS_ASSERTION(NS_FRAME_IS_COMPLETE(status), "bad status");
 
   // Place and size the child
-  metrics.Width() = aSize.width;
-  metrics.Height() = aSize.height;
-  FinishReflowChild(aChild, aPresContext, metrics, nullptr, aOffset.x, aOffset.y, 0);
+  reflowOutput.Width() = aSize.width;
+  reflowOutput.Height() = aSize.height;
+  FinishReflowChild(aChild, aPresContext, reflowOutput, nullptr, aOffset.x, aOffset.y, 0);
 }
 
 static
@@ -797,26 +792,26 @@ nscolor nsHTMLFramesetFrame::GetBorderColor(nsIContent* aContent)
 
 void
 nsHTMLFramesetFrame::Reflow(nsPresContext*           aPresContext,
-                            nsHTMLReflowMetrics&     aDesiredSize,
-                            const nsHTMLReflowState& aReflowState,
+                            ReflowOutput&     aDesiredSize,
+                            const ReflowInput& aReflowInput,
                             nsReflowStatus&          aStatus)
 {
   MarkInReflow();
   DO_GLOBAL_REFLOW_COUNT("nsHTMLFramesetFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
+  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   nsIPresShell *shell = aPresContext->PresShell();
   StyleSetHandle styleSet = shell->StyleSet();
 
   GetParent()->AddStateBits(NS_FRAME_CONTAINS_RELATIVE_BSIZE);
 
-  //printf("FramesetFrame2::Reflow %X (%d,%d) \n", this, aReflowState.AvailableWidth(), aReflowState.AvailableHeight());
+  //printf("FramesetFrame2::Reflow %X (%d,%d) \n", this, aReflowInput.AvailableWidth(), aReflowInput.AvailableHeight());
   // Always get the size so that the caller knows how big we are
-  GetDesiredSize(aPresContext, aReflowState, aDesiredSize);
+  GetDesiredSize(aPresContext, aReflowInput, aDesiredSize);
 
-  nscoord width  = (aDesiredSize.Width() <= aReflowState.AvailableWidth())
-    ? aDesiredSize.Width() : aReflowState.AvailableWidth();
-  nscoord height = (aDesiredSize.Height() <= aReflowState.AvailableHeight())
-    ? aDesiredSize.Height() : aReflowState.AvailableHeight();
+  nscoord width  = (aDesiredSize.Width() <= aReflowInput.AvailableWidth())
+    ? aDesiredSize.Width() : aReflowInput.AvailableWidth();
+  nscoord height = (aDesiredSize.Height() <= aReflowInput.AvailableHeight())
+    ? aDesiredSize.Height() : aReflowInput.AvailableHeight();
 
   // We might be reflowed more than once with NS_FRAME_FIRST_REFLOW;
   // that's allowed.  (Though it will only happen for misuse of frameset
@@ -850,7 +845,7 @@ nsHTMLFramesetFrame::Reflow(nsPresContext*           aPresContext,
   if (mNumRows != rows || mNumCols != cols) {
     aStatus = NS_FRAME_COMPLETE;
     mDrag.UnSet();
-    NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
+    NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aDesiredSize);
     return;
   }
 
@@ -928,7 +923,7 @@ nsHTMLFramesetFrame::Reflow(nsPresContext*           aPresContext,
         borderChildX++;
       }
       nsSize borderSize(aDesiredSize.Width(), borderWidth);
-      ReflowPlaceChild(borderFrame, aPresContext, aReflowState, offset, borderSize);
+      ReflowPlaceChild(borderFrame, aPresContext, aReflowInput, offset, borderSize);
       borderFrame = nullptr;
       offset.y += borderWidth;
     } else {
@@ -958,14 +953,14 @@ nsHTMLFramesetFrame::Reflow(nsPresContext*           aPresContext,
             borderChildX++;
           }
           nsSize borderSize(borderWidth, aDesiredSize.Height());
-          ReflowPlaceChild(borderFrame, aPresContext, aReflowState, offset, borderSize);
+          ReflowPlaceChild(borderFrame, aPresContext, aReflowInput, offset, borderSize);
           borderFrame = nullptr;
         }
         offset.x += borderWidth;
       }
     }
 
-    ReflowPlaceChild(child, aPresContext, aReflowState, offset, size, &cellIndex);
+    ReflowPlaceChild(child, aPresContext, aReflowInput, offset, size, &cellIndex);
 
     if (firstTime) {
       int32_t childVis;
@@ -983,8 +978,10 @@ nsHTMLFramesetFrame::Reflow(nsPresContext*           aPresContext,
           childVis = (eFrameborder_No == frameborder) ? NONE_VIS : ALL_VIS;
         }
       } else {  // blank
-        DebugOnly<nsHTMLFramesetBlankFrame*> blank;
-        MOZ_ASSERT(blank = do_QueryFrame(child), "unexpected child frame type");
+#ifdef DEBUG
+        nsHTMLFramesetBlankFrame* blank = do_QueryFrame(child);
+        MOZ_ASSERT(blank, "unexpected child frame type");
+#endif
         childVis = NONE_VIS;
       }
       nsBorderColor childColors = mChildBorderColors[childX];
@@ -1085,7 +1082,7 @@ nsHTMLFramesetFrame::Reflow(nsPresContext*           aPresContext,
   aDesiredSize.SetOverflowAreasToDesiredBounds();
   FinishAndStoreOverflow(&aDesiredSize);
 
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aDesiredSize);
 }
 
 nsIAtom*
@@ -1375,16 +1372,16 @@ void nsHTMLFramesetBorderFrame::SetColor(nscolor aColor)
 
 void
 nsHTMLFramesetBorderFrame::Reflow(nsPresContext*           aPresContext,
-                                  nsHTMLReflowMetrics&     aDesiredSize,
-                                  const nsHTMLReflowState& aReflowState,
+                                  ReflowOutput&     aDesiredSize,
+                                  const ReflowInput& aReflowInput,
                                   nsReflowStatus&          aStatus)
 {
   DO_GLOBAL_REFLOW_COUNT("nsHTMLFramesetBorderFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
+  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
 
   // Override Reflow(), since we don't want to deal with what our
   // computed values are.
-  SizeToAvailSize(aReflowState, aDesiredSize);
+  SizeToAvailSize(aReflowInput, aDesiredSize);
 
   aDesiredSize.SetOverflowAreasToDesiredBounds();
   aStatus = NS_FRAME_COMPLETE;
@@ -1590,15 +1587,15 @@ nscoord nsHTMLFramesetBlankFrame::GetIntrinsicBSize()
 
 void
 nsHTMLFramesetBlankFrame::Reflow(nsPresContext*           aPresContext,
-                                 nsHTMLReflowMetrics&     aDesiredSize,
-                                 const nsHTMLReflowState& aReflowState,
+                                 ReflowOutput&     aDesiredSize,
+                                 const ReflowInput& aReflowInput,
                                  nsReflowStatus&          aStatus)
 {
   DO_GLOBAL_REFLOW_COUNT("nsHTMLFramesetBlankFrame");
 
   // Override Reflow(), since we don't want to deal with what our
   // computed values are.
-  SizeToAvailSize(aReflowState, aDesiredSize);
+  SizeToAvailSize(aReflowInput, aDesiredSize);
 
   aDesiredSize.SetOverflowAreasToDesiredBounds();
   aStatus = NS_FRAME_COMPLETE;
