@@ -465,37 +465,6 @@ nsSVGPaintingProperty::DoUpdate()
   }
 }
 
-static nsSVGRenderingObserver *
-CreateMarkerProperty(nsIURI *aURI, nsIFrame *aFrame, bool aReferenceImage)
-{ return new nsSVGMarkerProperty(aURI, aFrame, aReferenceImage); }
-
-static nsSVGRenderingObserver *
-CreateTextPathProperty(nsIURI *aURI, nsIFrame *aFrame, bool aReferenceImage)
-{ return new nsSVGTextPathProperty(aURI, aFrame, aReferenceImage); }
-
-static nsSVGRenderingObserver *
-CreatePaintingProperty(nsIURI *aURI, nsIFrame *aFrame, bool aReferenceImage)
-{ return new nsSVGPaintingProperty(aURI, aFrame, aReferenceImage); }
-
-static nsSVGRenderingObserver *
-GetEffectProperty(nsIURI *aURI, nsIFrame *aFrame,
-                  nsSVGEffects::ObserverPropertyDescriptor aProperty,
-                  nsSVGRenderingObserver * (* aCreate)(nsIURI *, nsIFrame *, bool))
-{
-  if (!aURI)
-    return nullptr;
-
-  FrameProperties props = aFrame->Properties();
-  nsSVGRenderingObserver *prop =
-    static_cast<nsSVGRenderingObserver*>(props.Get(aProperty));
-  if (prop)
-    return prop;
-  prop = aCreate(aURI, aFrame, false);
-  NS_ADDREF(prop);
-  props.Set(aProperty, static_cast<nsISupports*>(prop));
-  return prop;
-}
-
 static nsSVGFilterProperty*
 GetOrCreateFilterProperty(nsIFrame *aFrame)
 {
@@ -527,37 +496,51 @@ GetOrCreateMaskProperty(nsIFrame *aFrame)
   return prop;
 }
 
+template<class T>
+static T*
+GetEffectProperty(nsIURI *aURI, nsIFrame *aFrame,
+  const mozilla::FramePropertyDescriptor<T>* aProperty)
+{
+  if (!aURI)
+    return nullptr;
+
+  FrameProperties props = aFrame->Properties();
+  T* prop = props.Get(aProperty);
+  if (prop)
+    return prop;
+  prop = new T(aURI, aFrame, false);
+  NS_ADDREF(prop);
+  props.Set(aProperty, prop);
+  return prop;
+}
+
 nsSVGMarkerProperty *
 nsSVGEffects::GetMarkerProperty(nsIURI *aURI, nsIFrame *aFrame,
-                                ObserverPropertyDescriptor aProp)
+  const mozilla::FramePropertyDescriptor<nsSVGMarkerProperty>* aProperty)
 {
   MOZ_ASSERT(aFrame->GetType() == nsGkAtoms::svgPathGeometryFrame &&
-               static_cast<nsSVGPathGeometryElement*>(aFrame->GetContent())->IsMarkable(),
+             static_cast<nsSVGPathGeometryElement*>(aFrame->GetContent())->IsMarkable(),
              "Bad frame");
-  return static_cast<nsSVGMarkerProperty*>(
-          GetEffectProperty(aURI, aFrame, aProp, CreateMarkerProperty));
+  return GetEffectProperty(aURI, aFrame, aProperty);
 }
 
 nsSVGTextPathProperty *
 nsSVGEffects::GetTextPathProperty(nsIURI *aURI, nsIFrame *aFrame,
-                                  ObserverPropertyDescriptor aProp)
+  const mozilla::FramePropertyDescriptor<nsSVGTextPathProperty>* aProperty)
 {
-  return static_cast<nsSVGTextPathProperty*>(
-          GetEffectProperty(aURI, aFrame, aProp, CreateTextPathProperty));
+  return GetEffectProperty(aURI, aFrame, aProperty);
 }
 
 nsSVGPaintingProperty *
 nsSVGEffects::GetPaintingProperty(nsIURI *aURI, nsIFrame *aFrame,
-                                  ObserverPropertyDescriptor aProp)
+  const mozilla::FramePropertyDescriptor<nsSVGPaintingProperty>* aProperty)
 {
-  return static_cast<nsSVGPaintingProperty*>(
-          GetEffectProperty(aURI, aFrame, aProp, CreatePaintingProperty));
+  return GetEffectProperty(aURI, aFrame, aProperty);
 }
 
-static nsSVGRenderingObserver *
-GetEffectPropertyForURI(nsIURI *aURI, nsIFrame *aFrame,
-                        nsSVGEffects::URIObserverHashtablePropertyDescriptor aProperty,
-                        nsSVGRenderingObserver * (* aCreate)(nsIURI *, nsIFrame *, bool))
+nsSVGPaintingProperty *
+nsSVGEffects::GetPaintingPropertyForURI(nsIURI *aURI, nsIFrame *aFrame,
+  URIObserverHashtablePropertyDescriptor aProperty)
 {
   if (!aURI)
     return nullptr;
@@ -568,24 +551,16 @@ GetEffectPropertyForURI(nsIURI *aURI, nsIFrame *aFrame,
     hashtable = new nsSVGEffects::URIObserverHashtable();
     props.Set(aProperty, hashtable);
   }
-  nsSVGRenderingObserver* prop =
-    static_cast<nsSVGRenderingObserver*>(hashtable->GetWeak(aURI));
+  nsSVGPaintingProperty* prop =
+    static_cast<nsSVGPaintingProperty*>(hashtable->GetWeak(aURI));
   if (!prop) {
     bool watchImage = aProperty == nsSVGEffects::BackgroundImageProperty();
-    prop = aCreate(aURI, aFrame, watchImage);
+    prop = new nsSVGPaintingProperty(aURI, aFrame, watchImage);
     hashtable->Put(aURI, prop);
   }
   return prop;
 }
 
-nsSVGPaintingProperty *
-nsSVGEffects::GetPaintingPropertyForURI(nsIURI *aURI, nsIFrame *aFrame,
-                                        URIObserverHashtablePropertyDescriptor aProp)
-{
-  return static_cast<nsSVGPaintingProperty*>(
-          GetEffectPropertyForURI(aURI, aFrame, aProp, CreatePaintingProperty));
-}
-
 nsSVGEffects::EffectProperties
 nsSVGEffects::GetEffectProperties(nsIFrame *aFrame)
 {
@@ -596,7 +571,7 @@ nsSVGEffects::GetEffectProperties(nsIFrame *aFrame)
 
   result.mFilter = GetOrCreateFilterProperty(aFrame);
 
-  if (style->mClipPath.GetType() == NS_STYLE_CLIP_PATH_URL) {
+  if (style->mClipPath.GetType() == StyleClipPathType::URL) {
     result.mClipPath =
       GetPaintingProperty(style->mClipPath.GetURL(), aFrame, ClipPathProperty());
   } else {
@@ -612,7 +587,7 @@ nsSVGEffects::GetEffectProperties(nsIFrame *aFrame)
 
 nsSVGPaintServerFrame *
 nsSVGEffects::GetPaintServer(nsIFrame *aTargetFrame, const nsStyleSVGPaint *aPaint,
-                             ObserverPropertyDescriptor aType)
+                             PaintingPropertyDescriptor aType)
 {
   if (aPaint->mType != eStyleSVGPaintType_Server)
     return nullptr;
@@ -720,12 +695,9 @@ nsSVGEffects::UpdateEffects(nsIFrame *aFrame)
       static_cast<nsSVGPathGeometryElement*>(aFrame->GetContent())->IsMarkable()) {
     // Set marker properties here to avoid reference loops
     const nsStyleSVG *style = aFrame->StyleSVG();
-    GetEffectProperty(style->mMarkerStart, aFrame, MarkerBeginProperty(),
-                      CreateMarkerProperty);
-    GetEffectProperty(style->mMarkerMid, aFrame, MarkerMiddleProperty(),
-                      CreateMarkerProperty);
-    GetEffectProperty(style->mMarkerEnd, aFrame, MarkerEndProperty(),
-                      CreateMarkerProperty);
+    GetMarkerProperty(style->mMarkerStart, aFrame, MarkerBeginProperty());
+    GetMarkerProperty(style->mMarkerMid, aFrame, MarkerMiddleProperty());
+    GetMarkerProperty(style->mMarkerEnd, aFrame, MarkerEndProperty());
   }
 }
 
@@ -737,8 +709,7 @@ nsSVGEffects::GetFilterProperty(nsIFrame *aFrame)
   if (!aFrame->StyleEffects()->HasFilters())
     return nullptr;
 
-  return static_cast<nsSVGFilterProperty *>
-    (aFrame->Properties().Get(FilterProperty()));
+  return aFrame->Properties().Get(FilterProperty());
 }
 
 void
