@@ -8,7 +8,6 @@
 
 #include "gfxMatrix.h"
 #include "gfxRect.h"
-#include "nsAutoPtr.h"
 #include "nsRegionFwd.h"
 
 class gfxContext;
@@ -38,6 +37,7 @@ struct nsSize;
 class nsSVGIntegrationUtils final
 {
   typedef mozilla::gfx::DrawTarget DrawTarget;
+  typedef mozilla::image::DrawResult DrawResult;
 
 public:
   /**
@@ -130,21 +130,23 @@ public:
     const nsRect& borderArea;
     nsDisplayListBuilder* builder;
     mozilla::layers::LayerManager* layerManager;
+    bool callerPaintsOpacity;
     explicit PaintFramesParams(gfxContext& aCtx, nsIFrame* aFrame,
                                const nsRect& aDirtyRect,
                                const nsRect& aBorderArea,
                                nsDisplayListBuilder* aBuilder,
-                               mozilla::layers::LayerManager* aLayerManager)
+                               mozilla::layers::LayerManager* aLayerManager,
+                               bool aCallerPaintsOpacity)
       : ctx(aCtx), frame(aFrame), dirtyRect(aDirtyRect),
         borderArea(aBorderArea), builder(aBuilder),
-        layerManager(aLayerManager)
+        layerManager(aLayerManager), callerPaintsOpacity(aCallerPaintsOpacity)
     { }
   };
 
   /**
    * Paint non-SVG frame with SVG effects.
    */
-  static void
+  static DrawResult
   PaintFramesWithEffects(const PaintFramesParams& aParams);
 
   /**
