@@ -7,11 +7,25 @@
 #ifndef MOZ_FFVPX_CONFIG_H
 #define MOZ_FFVPX_CONFIG_H
 #if defined(XP_WIN)
+// Avoid conflicts with mozilla-config.h
+#if !defined(_MSC_VER)
+#undef HAVE_DIRENT_H
+#undef HAVE_UNISTD_H
+#endif
 #if defined(HAVE_64BIT_BUILD)
 #include "config_win64.h"
 #else
 #include "config_win32.h"
 #endif
+// Adjust configure defines for GCC
+#if !defined(_MSC_VER)
+#if !defined(HAVE_64BIT_BUILD)
+#undef HAVE_MM_EMPTY
+#define HAVE_MM_EMPTY 0
+#endif
+#undef HAVE_LIBC_MSVCRT
+#define HAVE_LIBC_MSVCRT 0
+#endif
 #elif defined(XP_DARWIN)
 #if defined(HAVE_64BIT_BUILD)
 #include "config_darwin64.h"
