@@ -17,6 +17,9 @@
 #include <algorithm>
 #include "nsContentSecurityManager.h"
 #include "nsHttp.h"
+#include "nsNetUtil.h"
+#include "nsIURI.h"
+#include "nsHttpHeaderArray.h"
 
 //
 // Helper function for determining the length of data bytes up to
@@ -418,7 +421,8 @@ nsPartChannel::VisitResponseHeaders(nsIHttpHeaderVisitor *visitor)
 {
     if (!mResponseHead)
         return NS_ERROR_NOT_AVAILABLE;
-    return mResponseHead->Headers().VisitHeaders(visitor);
+    return mResponseHead->VisitHeaders(visitor,
+        mozilla::net::nsHttpHeaderArray::eFilterResponse);
 }
 
 //
@@ -664,6 +668,10 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
                     // Push the cursor to the token so that the while loop below will
                     // find token from the right position.
                     cursor = tokenPos;
+
+                    // Update bufLen to exlude the preamble. Otherwise, the first
+                    // |SendData| would claim longer buffer length.
+                    bufLen -= mPreamble.Length();
                 }
             } else {
                 // If the boundary was set in the header,
@@ -1207,9 +1215,8 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
         *newLine = '\0'; // cursor is now null terminated
 
         if (mResponseHead) {
-            // ParseHeaderLine is destructive. We create a copy
             nsAutoCString tmpHeader(cursor);
-            mResponseHead->ParseHeaderLine(tmpHeader.get());
+            mResponseHead->ParseHeaderLine(tmpHeader);
         }
 
         char *colon = (char *) strchr(cursor, ':');
@@ -1342,4 +1349,3 @@ NS_NewMultiMixedConv(nsMultiMixedConv** aMultiMixedConv)
     NS_ADDREF(*aMultiMixedConv);
     return NS_OK;
 }
-
