@@ -31,10 +31,10 @@ LiteralInput(const char(&valueString)[N])
 
 const int MaxCacheEntries = 1024;
 
-class OCSPCacheTest : public ::testing::Test
+class psm_OCSPCacheTest : public ::testing::Test
 {
 protected:
-  OCSPCacheTest() : now(Now()) { }
+  psm_OCSPCacheTest() : now(Now()) { }
 
   static void SetUpTestCase()
   {
@@ -70,7 +70,7 @@ Input fakeKey000(LiteralInput("key000"));
 Input fakeKey001(LiteralInput("key001"));
 Input fakeSerial0000(LiteralInput("0000"));
 
-TEST_F(OCSPCacheTest, TestPutAndGet)
+TEST_F(psm_OCSPCacheTest, TestPutAndGet)
 {
   Input fakeSerial000(LiteralInput("000"));
   Input fakeSerial001(LiteralInput("001"));
@@ -84,7 +84,7 @@ TEST_F(OCSPCacheTest, TestPutAndGet)
                          resultOut, timeOut));
 }
 
-TEST_F(OCSPCacheTest, TestVariousGets)
+TEST_F(psm_OCSPCacheTest, TestVariousGets)
 {
   SCOPED_TRACE("");
   for (int i = 0; i < MaxCacheEntries; i++) {
@@ -132,7 +132,7 @@ TEST_F(OCSPCacheTest, TestVariousGets)
                          resultOut, timeOut));
 }
 
-TEST_F(OCSPCacheTest, TestEviction)
+TEST_F(psm_OCSPCacheTest, TestEviction)
 {
   SCOPED_TRACE("");
   // By putting more distinct entries in the cache than it can hold,
@@ -155,7 +155,7 @@ TEST_F(OCSPCacheTest, TestEviction)
                          resultOut, timeOut));
 }
 
-TEST_F(OCSPCacheTest, TestNoEvictionForRevokedResponses)
+TEST_F(psm_OCSPCacheTest, TestNoEvictionForRevokedResponses)
 {
   SCOPED_TRACE("");
   CertID notEvicted(fakeIssuer1, fakeKey000, fakeSerial0000);
@@ -185,7 +185,7 @@ TEST_F(OCSPCacheTest, TestNoEvictionForRevokedResponses)
   ASSERT_FALSE(cache.Get(evicted, resultOut, timeOut));
 }
 
-TEST_F(OCSPCacheTest, TestEverythingIsRevoked)
+TEST_F(psm_OCSPCacheTest, TestEverythingIsRevoked)
 {
   SCOPED_TRACE("");
   Time timeIn(now);
@@ -228,7 +228,7 @@ TEST_F(OCSPCacheTest, TestEverythingIsRevoked)
   ASSERT_EQ(Result::ERROR_REVOKED_CERTIFICATE, result);
 }
 
-TEST_F(OCSPCacheTest, VariousIssuers)
+TEST_F(psm_OCSPCacheTest, VariousIssuers)
 {
   SCOPED_TRACE("");
   Time timeIn(now);
@@ -249,7 +249,7 @@ TEST_F(OCSPCacheTest, VariousIssuers)
                          resultOut, timeOut));
 }
 
-TEST_F(OCSPCacheTest, Times)
+TEST_F(psm_OCSPCacheTest, Times)
 {
   SCOPED_TRACE("");
   CertID certID(fakeIssuer1, fakeKey000, fakeSerial0000);
@@ -273,7 +273,7 @@ TEST_F(OCSPCacheTest, Times)
             TimeFromElapsedSecondsAD(50));
 }
 
-TEST_F(OCSPCacheTest, NetworkFailure)
+TEST_F(psm_OCSPCacheTest, NetworkFailure)
 {
   SCOPED_TRACE("");
   CertID certID(fakeIssuer1, fakeKey000, fakeSerial0000);
