@@ -1,6 +1,6 @@
 "use strict";
 
-extensions.registerSchemaAPI("extension", null, (extension, context) => {
+extensions.registerSchemaAPI("extension", (extension, context) => {
   return {
     extension: {
       getURL: function(url) {
@@ -11,6 +11,10 @@ extensions.registerSchemaAPI("extension", null, (extension, context) => {
         let result = Cu.cloneInto([], context.cloneScope);
 
         for (let view of extension.views) {
+          if (!context.principal.subsumes(view.principal)) {
+            continue;
+          }
+
           if (fetchProperties !== null) {
             if (fetchProperties.type !== null && view.type != fetchProperties.type) {
               continue;
