@@ -32,7 +32,7 @@ function* promiseKeyword(keyword, expectedHref) {
     yield new Promise(resolve => do_timeout(100, resolve));
     let entry = yield PlacesUtils.keywords.fetch(keyword);
     if (entry)
-      href = entry.url.href; 
+      href = entry.url.href;
   } while (href != expectedHref);
 }
 
@@ -56,7 +56,7 @@ function expectBookmarkNotifications() {
       }
 
       if (name.startsWith("onItemChanged")) {
-        return (itemId, property) => {
+        return function(itemId, property) {
           if (property != "keyword")
             return;
           let args = Array.from(arguments, arg => {
